
CREATE TABLE persones (
	dni char(8),
	nom char(30),
	PRIMARY KEY (dni));

CREATE TABLE funcionaris (
	dni CHAR(8),
	nss INTEGER,
	sou INTEGER,
	PRIMARY KEY (dni),
	FOREIGN KEY (dni) REFERENCES persones);

CREATE TABLE pescadors (
	dni CHAR(8),
	carrer CHAR(50),
	ciutat CHAR(20),
	comunitat CHAR(25),
	PRIMARY KEY (dni),
	FOREIGN KEY (dni) REFERENCES persones);

CREATE TABLE tipus_zona (
	id INTEGER UNIQUE,
	tipus CHAR(40),
	PRIMARY KEY (tipus));

CREATE TABLE zones (
	num_zona INTEGER,
	nom_massa CHAR(20),
	municipi CHAR(20),
	tipus_zona CHAR(40),
	PRIMARY KEY (num_zona, nom_massa),
	FOREIGN KEY (tipus_zona) REFERENCES tipus_zona);
	
CREATE TABLE especies (
	nom_popular CHAR(20),
	nom_cientific CHAR(20),
	long_mitja INTEGER,
	PRIMARY KEY (nom_popular),
	UNIQUE (nom_cientific));

CREATE TABLE assignacions (
	funcionari CHAR(8),
	data_inici DATE,
	num_zona INTEGER,
	nom_massa CHAR(20),
	data_fi DATE,
	PRIMARY KEY (funcionari, num_zona, nom_massa, data_inici),
	FOREIGN KEY (funcionari) REFERENCES funcionaris,
	FOREIGN KEY (num_zona, nom_massa) REFERENCES zones(num_zona, nom_massa));

CREATE TRIGGER dates AFTER INSERT OR UPDATE OF data_inici,data_fi ON assignacions
FOR EACH ROW 
    
BEGIN
    IF (:new.data_inici > :new.data_fi ) THEN
        RAISE_APPLICATION_ERROR(-20000, 'Les dates no son coherents');
    END IF; 
END;
.
RUN; 


CREATE TABLE permisos (
	num_zona INTEGER,
	nom_massa CHAR(20),
	data_vigencia DATE,
	num_max_permisos INTEGER CHECK (num_max_permisos < 51),
	PRIMARY KEY (num_zona, nom_massa,data_vigencia),
	FOREIGN KEY (num_zona, nom_massa) REFERENCES zones(num_zona, nom_massa));

CREATE TRIGGER zones_permisos AFTER INSERT OR UPDATE OF num_zona,nom_massa ON permisos
FOR EACH ROW 
DECLARE

    l NUMBER;
    
BEGIN
    SELECT T.id INTO l FROM zones Z, tipus_zona T WHERE Z.tipus_zona=T.tipus AND Z.num_zona=:new.num_zona AND Z.nom_massa=:new.nom_massa;
    IF ((l=1) OR (l=2)) THEN
        RAISE_APPLICATION_ERROR(-20000, 'Zona no permesa');
    END IF; 
END;
.
RUN; 


CREATE TABLE captures (
	num_zona INTEGER,
	nom_massa CHAR(20),
	nom_especie CHAR(20),
	num_max_captures INTEGER,
	long_mitja_captures INTEGER,
	PRIMARY KEY (num_zona,nom_massa,nom_especie),
	FOREIGN KEY (num_zona, nom_massa) REFERENCES zones(num_zona, nom_massa),
	FOREIGN KEY (nom_especie) REFERENCES especies);

CREATE TRIGGER longitud AFTER INSERT OR UPDATE OF long_mitja_captures ON captures
FOR EACH ROW 
DECLARE

    l NUMBER;
    
BEGIN
    SELECT long_mitja INTO l FROM especies WHERE nom_popular=:new.nom_especie;
    IF (:new.long_mitja_captures > l) THEN
        RAISE_APPLICATION_ERROR(-20000, 'No es permet una longitud de captura mitja per sobre de la longitud mitja de la especie');
    END IF; 
END;
.
RUN; 

CREATE TRIGGER zones_captures AFTER INSERT OR UPDATE OF num_zona,nom_massa ON captures
FOR EACH ROW 
DECLARE

    l NUMBER;
    
BEGIN
    SELECT T.id INTO l FROM zones Z, tipus_zona T WHERE Z.tipus_zona=T.tipus AND Z.num_zona=:new.num_zona AND Z.nom_massa=:new.nom_massa;
    IF ((l=1) OR (l=2) OR (l=3)) THEN
        RAISE_APPLICATION_ERROR(-20000, 'Zona no permesa');
    END IF; 
END;
.
RUN; 



CREATE TABLE multes (
	funcionari CHAR(8),
	infractor CHAR(8),
	data_multa DATE,
	num_zona INTEGER,
	nom_massa CHAR(20),
	motiu CHAR(50),
	import INTEGER,
	PRIMARY KEY (funcionari,infractor,data_multa,num_zona,nom_massa),
	FOREIGN KEY (funcionari) REFERENCES funcionaris,
	FOREIGN KEY (infractor) REFERENCES pescadors);

CREATE TABLE habitats (
	num_zona INTEGER,
	nom_massa CHAR(20),
	especie CHAR(20),
	index_poblacio INTEGER,
	PRIMARY KEY (num_zona, nom_massa, especie),
	FOREIGN KEY (num_zona, nom_massa) REFERENCES zones(num_zona, nom_massa),
	FOREIGN KEY (especie) REFERENCES especies);

##falta en multes
##	FOREIGN KEY (num_zona, nom_massa) REFERENCES zones(num_zona, nom_massa)
##